# BomberMan release 3 by David Bowland
# ./addons/eventscripts/bomberman/bomberman.py

# >>> To configure this addon please see bomberman.cfg <<<

"""
Makes players without a kill in certain number of rounds or deaths explosive. Requires only ES 2.0+
"""


import es
import gamethread
import langlib
import os.path
import playerlib
import random


info = es.AddonInfo()
info.name = 'BomberMan'
info.version = '3'
info.url = 'http://addons.eventscripts.com/addons/view/bomberman'
info.basename = 'bomberman'
info.author = 'SuperDave'


int_noisy = 0
int_added_prefix = 0
list_recently_created = []
dict_players = {}
str_last_sound = ''

tuple_sounds = ('radio/blow.wav', 'bot/the_bombs_here.wav', 'bot/theres_the_bomb.wav', 'bot/theres_the_bomb2.wav', 'bot/theres_the_bomber.wav', 'bot/they_got_the_bomb.wav', 'bot/they_picked_up_the_bomb.wav')
dict_bomber_colors = {0:{'string':'255 255 255', 'int':16777215}, 1:{'string':'255 0 0', 'int':255}, 2:{'string':'0 0 255', 'int':16711680}, 3:{'string':'0 255 0', 'int':65280}, 4:{'string':'255 255 0', 'int':65535}, 5:{'string':'100 50 0', 'int':12900}, 6:{'string':'60 60 60', 'int':3947580}, 7:{'string':'0 0 0', 'int':0}}
dict_options = {'bomberman_rounds':[4, 'Number of rounds with no kills required to become a bomber--set to 0 to remove this option'], 'bomberman_deaths':[4, 'Number of deaths with no kills required to become a bomber--set to 0 to remove this option'] ,'bomberman_allowbot':[0, '0 = bots are not allowed to be bombers, 1 = bots are allowed to be bombers'], 'bomberman_bomber_trigger_hurt':[1, '0 = no change, 1 = bomber players explode when hurt'], 'bomberman_bomber_trigger_fire':[1, '0 = no change, 1 = bomber players explode when firing a weapon'], 'bomberman_bomber_trigger_delay':[5, 'Number of seconds after spawning until a bomber player can trigger a bomb with weapon fire--set to 0 for no delay'], 'bomberman_bomber_speed':[150, 'Percent of normal speed bomber players travel'], 'bomberman_bomber_color':[1, 'Color of players with bomb: 0 = no change, 1 = red, 2 = blue, 3 = green, 4 = yellow, 5 = brown, 6 = gray, 7 = black'], 'bomberman_bomber_transparency':[255, 'Transparency of bomber players from 0 (invisible) to 255 (opaque/no change)'], 'bomberman_bomber_wearsoff':[0, '0 = players remian a bomber until they get a kill, 1 = player will only be bomber for one round'] ,'bomberman_bomber_sound':[50, 'Percent of the map that can hear when a player changes to a bomber'], 'bomberman_explosion_radius':[500, 'Radius of bomber explosion--HE grenade is 350'], 'bomberman_explosion_magnitude':[500, 'Magniture of bomber explosion--HE grenade is 100'] ,'bomberman_explosion_sound':[30, 'Percent of the map that can hear when a bomber explodes'], 'bomberman_announce_rounds':[2, '0 = no change, 1 = players receive a console message with round count information, 2 = players receive a chat area message with round count information'], 'bomberman_announce_deaths':[2, '0 = no change, 1 = players receive a console message with death count information, 2 = players receive a chat area message with death count information'], 'bomberman_announce_bomber_hud':[1, '0 = no change, 1 = players receive a HUD message when they become a bomber'], 'bomberman_announce_bomber_center':[1, '0 = no change, 1 = players receive a center text message when they become a bomber'], 'bomberman_announce_bomber_chat':[3, '0 = no change, 1 = players receive a chat area message when they become a bomber, 2 = players on the same team as the bomber receive a chat area message when a player becomes a bomber, 3 = all players receive a chat area message when a player becomes a bomber'], 'bomberman_announce_bomber_console':[0, '0 = no change, 1 = players receive a console message when they become a bomber, 2 = players on the same team as the bomber receive a console message when a player becomes a bomber, 3 = all players receive a console message when a player becomes a bomber'], 'bomberman_announce_bomber_trigger_chat':[1, '0 = no change, 1 = players receive a chat area message when they become a bomber on how to trigger the bomb'], 'bomberman_announce_bomber_trigger_console':[0, '0 = no change, 1 = players receive a console message when they become a bomber on how to trigger the bomb'], 'bomberman_announce_bomber_triggererror_hud':[1, '0 = no change, 1 = players receive a HUD message when they try to trigger the bomb too early'], 'bomberman_announce_bomber_triggererror_center':[1, '0 = no change, 1 = players receive a center text message when they try to trigger the bomb too early'], 'bomberman_announce_bomber_triggererror_chat':[0, '0 = no change, 1 = players receive a chat area message when they try to trigger the bomb too early']}
func_lang_text = None


def load():
   """
   Checks to make sure critical server variables are created by the config
   Calls player_spawn for each player
   Activates eventscripts_noisy
   Adds 'd' and 'D' to eventscripts_cmdprefixes
   """
   global int_noisy
   global int_added_prefix
   global func_lang_text

   for str_option in dict_options:
      es.ServerVar(str_option, dict_options[str_option][0], dict_options[str_option][1])
   if os.path.isfile(es.getAddonPath('bomberman') + '/bomberman.cfg'):
      es.server.cmd('es_xmexec ../addons/eventscripts/bomberman/bomberman.cfg')
   else:
      es.dbgmsg(0, 'BomberMan: Unable to load bomberman.cfg! Please ensure it is in the ./bomberman/ directory.')

   str_lang_path = es.getAddonPath('bomberman') + '/bomberman_languages.ini'
   if os.path.isfile(str_lang_path):
      func_lang_text = langlib.Strings(str_lang_path)
   else:
      es.dbgmsg(0, 'BomberMan: Unable to load bomberman_languages.ini! Please ensure it is in the ./bomberman/ directory.')
   if not func_lang_text:
      func_lang_text = missing_lang

   for int_userid in es.getUseridList():
      player_spawn({'userid':str(int_userid), 'es_userteam':str(es.getplayerteam(int_userid))})

   if int(es.ServerVar('bomberman_bomber_trigger_fire')):
      es.doblock('corelib/noisy_on')
      if int(es.ServerVar('eventscripts_noisy')):
         int_noisy = 2
         es.ServerVar('eventscripts_noisy').set(1)
      else:
         int_noisy = 1

   if 'd' not in str(es.ServerVar('eventscripts_cmdprefixes')) or 'D' not in str(es.ServerVar('eventscripts_cmdprefixes')):
      int_added_prefix = 1
      es.ServerVar('eventscripts_cmdprefixes').set(es.ServerVar('eventscripts_cmdprefixes') + 'dD')


def es_map_start(event_var):
   """Clears the list of recently created players and the dictionary of players"""
   global list_recently_created
   global dict_players

   list_recently_created[:] = []
   dict_players.clear()


def es_client_command(event_var):
   """Calls color_weapons if a bomber uses the "drop" command"""
   if event_var['command'] == 'drop':
      int_userid = int(event_var['userid'])
      if dict_players.has_key(int_userid):
         dict_current_player = dict_players[int_userid]
         if dict_current_player['is_bomber'] == 1:
            color_weapons(int_userid, dict_current_player['color'], dict_current_player['alpha'], 0)


def round_start(event_var):
   """
   Designates players without a kill in the specified numebr of rounds as bomber players
   Clears the list of recent created players
   """
   global list_recently_created
   global dict_players

   int_rounds_to_bomber = int(es.ServerVar('bomberman_rounds'))
   if int_rounds_to_bomber:
      int_announce_option = int(es.ServerVar('bomberman_announce_rounds'))
      dict_tokens = {'count':0, 'limit':int_rounds_to_bomber}
      for int_userid in dict_players:
         str_player_lang = playerlib.getPlayer(int_userid).get('lang')
         dict_current_player = dict_players[int_userid]
         dict_tokens['count'] = dict_current_player['round_count']
         if not dict_current_player['round_kills'] and int_userid not in list_recently_created:
            dict_current_player['round_count'] += 1
            dict_tokens['count'] += 1
            if dict_current_player['round_count'] >= int_rounds_to_bomber:
               dict_current_player['round_count'] = 0
               if not dict_current_player['is_bomber']:
                  make_bomber(int_userid)
         dict_current_player['round_kills'] = 0

         if int_announce_option == 2:
            es.tell(int_userid, '#multi', func_lang_text('rounds', dict_tokens, str_player_lang))
         elif int_announce_option:
            es.cexec(int_userid, 'echo ' + remove_tags(func_lang_text('rounds', dict_tokens, str_player_lang)))

   list_recently_created[:] = []


def player_spawn(event_var):
   """
   Creates the player in the dictionary
   Calls make_bomber if necessary
   """
   global list_recently_created
   global dict_players

   int_team = int(event_var['es_userteam'])
   if int_team > 1:
      int_userid = int(event_var['userid'])
      if not dict_players.has_key(int_userid):
         dict_players[int_userid] = {'round_kills':0, 'round_count':0, 'death_kills':0, 'death_count':0, 'is_bomber':0, 'speed':100, 'color':0, 'alpha':255, 'weapons':[]}
         list_recently_created.append(int_userid)

      dict_current_player = dict_players[int_userid]
      int_bomber = dict_current_player['is_bomber']
      if int_bomber:
         if int_bomber == 2:
            gamethread.cancelDelayed('bomberman_%s' % int_userid)
         make_bomber(int_userid)


def player_hurt(event_var):
   """Explodes the player if the player is a bomber"""
   global dict_players

   if int(es.ServerVar('bomberman_bomber_trigger_hurt')):
      int_userid = int(event_var['userid'])
      if dict_players[int_userid]['is_bomber'] == 1:
         es.sexec(int_userid, 'kill')


def player_death(event_var):
   """
   Cancels outstanding delays for the victim
   Designates the victim as a bomb if necessary
   Incerements the attacker's stats
   """
   global dict_players

   int_userid = int(event_var['userid'])
   dict_current_player = dict_players[int_userid]
   str_player_lang = playerlib.getPlayer(int_userid).get('lang')
   int_deaths_to_bomber = int(es.ServerVar('bomberman_deaths'))
   dict_tokens = {'count':dict_current_player['death_count'], 'limit':int_deaths_to_bomber}

   int_bomber = dict_current_player['is_bomber']
   if int_bomber == 2:
      gamethread.cancelDelayed('bomberman_%s' % int_userid)
   if int_bomber:
      if int(es.ServerVar('bomberman_bomber_wearsoff')):
         remove_bomber(int_userid)
      else:
         explode_bomber(int_userid)

   elif int_deaths_to_bomber:
      if not dict_current_player['death_kills']:
         dict_current_player['death_count'] += 1
         dict_tokens['count'] += 1
         if dict_current_player['death_count'] >= int_deaths_to_bomber:
            dict_current_player['death_count'] = 0
            if not dict_current_player['is_bomber'] and (event_var['es_steamid'] <> 'BOT' or int(es.ServerVar('bomberman_allowbot'))):
               dict_current_player['is_bomber'] = 1
      int_announce_option = int(es.ServerVar('bomberman_announce_deaths'))
      if int_announce_option == 2:
         es.tell(int_userid, '#multi', func_lang_text('deaths', dict_tokens, str_player_lang))
      elif int_announce_option:
         es.cexec(int_userid, 'echo ' + remove_tags(func_lang_text('deaths', dict_tokens, str_player_lang)))
   dict_current_player['death_kills'] = 0

   int_attacker = int(event_var['attacker'])
   if event_var['es_userteam'] <> event_var['es_attackerteam'] and int_attacker:
      dict_current_player = dict_players[int_attacker]
      dict_current_player['round_kills'] += 1
      dict_current_player['round_count'] = 0
      dict_current_player['death_kills'] += 1
      dict_current_player['death_count'] = 0
      if int(event_var['es_attackerdead']) and dict_current_player['is_bomber']:
         dict_current_player['is_bomber'] = 0


def player_team(event_var):
   """Calls player_disconnect"""
   if event_var['team'] == '1':
      player_disconnect(event_var)


def player_disconnect(event_var):
   """Removes the player from the dictionary and sets the color of the player's weapons to normal"""
   global dict_players

   int_userid = int(event_var['userid'])
   if dict_players.has_key(int_userid):
      int_bomber = dict_players[int_userid]['is_bomber']
      if int_bomber == 2:
         gamethread.cancelDelayed('bomberman_%s' % int_userid)
      elif int_bomber:
         dict_entity_list = es.createentitylist()
         for int_weapon_index in dict_players[int_userid]['weapons']:
            if dict_entity_list.has_key(int_weapon_index):
               es.setindexprop(int_weapon_index, 'CBaseEntity.m_nRenderMode', es.getindexprop(int_weapon_index, 'CBaseEntity.m_nRenderMode') | 1)
               es.setindexprop(int_weapon_index, 'CBaseEntity.m_nRenderFX', es.getindexprop(int_weapon_index, 'CBaseEntity.m_nRenderFX') | 256)
               es.setindexprop(int_weapon_index, 'CBaseEntity.m_clrRender', 4294967295)
      del dict_players[int_userid]


def item_pickup(event_var):
   """Calls color_weapons to color the picked up item if the player is a bomber"""
   int_userid = int(event_var['userid'])
   if dict_players.has_key(int_userid):
      dict_current_player = dict_players[int_userid]
      if dict_current_player['is_bomber'] == 1:
         color_weapons(int_userid, dict_current_player['color'], dict_current_player['alpha'], 0)


def weapon_fire(event_var):
   """Explodes the player if the player is a bomber"""
   global dict_players

   if int(es.ServerVar('bomberman_bomber_trigger_fire')):
      int_userid = int(event_var['userid'])
      int_bomber = dict_players[int_userid]['is_bomber']
      if int_bomber:
         if int_bomber == 2:
            str_player_lang = playerlib.getPlayer(int_userid).get('lang')
            if int(es.ServerVar('bomberman_announce_bomber_triggererror_hud')):
               es.server.cmd('es_xfire %s env_hudhint kill' % int_userid)
               es.server.cmd('es_xgive %s env_hudhint' % int_userid)
               es.server.cmd('es_xfire %s env_hudhint addoutput \"message %s\"' % (int_userid, remove_tags(func_lang_text('trigger error', {}, str_player_lang))))
               es.server.cmd('es_xfire %s env_hudhint showhudhint' % int_userid)
            if int(es.ServerVar('bomberman_announce_bomber_triggererror_center')):
               es.centertell(int_userid, remove_tags(func_lang_text('trigger error', {}, str_player_lang)))
            if int(es.ServerVar('bomberman_announce_bomber_triggererror_chat')):
               es.tell(int_userid, '#multi', func_lang_text('trigger error', {}, str_player_lang))
         else:
            es.sexec(int_userid, 'kill')


def round_end(event_var):
   """Removes bomber designation from players"""
   if event_var['reason'] == '16':
      es_map_start({})
   else:
      if int(es.ServerVar('bomberman_bomber_wearsoff')):
         for int_userid in dict_players:
            remove_bomber(int_userid)


def unload():
   """
   Removes eventscripts_noisy
   Removes 'd' and 'D' from eventscripts_cmdprefixes
   Cancels outstanding delays
   """
   if int_noisy == 2:
      es.ServerVar('eventscripts_noisy').set(0)
   elif int_noisy:
      es.doblock('corelib/noisy_off')

   if int_added_prefix:
      str_temp = str(es.ServerVar('eventscripts_cmdprefixes'))
      es.ServerVar('eventscripts_cmdprefixes').set(str_temp.replace('dD', ''))

   for int_userid in dict_players:
      int_bomber = dict_players[int_userid]['is_bomber']
      if int_bomber:
         remove_bomber(int_userid)
         if int_bomber == 2:
            gamethread.cancelDelayed('bomberman_%s' % int_userid)

   for str_option in dict_options:
      es.ServerVar(str_option).set(0)


def make_bomber(int_userid):
   """
   Calls finalize_bomber
   Announces to the player that he or she is a bomber and instructions on exploding the bomb
   """
   global dict_players

   if es.getplayersteamid(int_userid) <> 'BOT' or int(es.ServerVar('bomberman_allowbot')):
      float_delay = float(es.ServerVar('bomberman_bomber_trigger_delay'))
      if float_delay:
         dict_players[int_userid]['is_bomber'] = 2
         gamethread.delayedname(float_delay, 'bomberman_%s' % int_userid, finalize_bomber, int_userid)
      else:
         finalize_bomber(int_userid)

      str_player_lang = playerlib.getPlayer(int_userid).get('lang')
      dict_tokens = {'player':es.getplayername(int_userid)}
      int_team = es.getplayerteam(int_userid)
      if int(es.ServerVar('bomberman_announce_bomber_hud')):
         es.server.cmd('es_xfire %s env_hudhint kill' % int_userid)
         es.server.cmd('es_xgive %s env_hudhint' % int_userid)
         es.server.cmd('es_xfire %s env_hudhint addoutput \"message %s\"' % (int_userid, remove_tags(func_lang_text('bomber', {}, str_player_lang))))
         es.server.cmd('es_xfire %s env_hudhint showhudhint' % int_userid)
      if int(es.ServerVar('bomberman_announce_bomber_center')):
         es.centertell(int_userid, remove_tags(func_lang_text('bomber', {}, str_player_lang)))
      int_option = int(es.ServerVar('bomberman_announce_bomber_chat'))
      if int_option:
         es.tell(int_userid, '#multi', func_lang_text('bomber', {}, str_player_lang))
         if int_option in (2, 3):
            for int_loop_userid in es.getUseridList():
               if int_loop_userid <> int_userid and (int_option == 3 or int_team == es.getplayerteam(int_loop_userid)):
                  es.tell(int_loop_userid, '#multi', func_lang_text('bomber all', dict_tokens, playerlib.getPlayer(int_loop_userid).get('lang')))
      int_option = int(es.ServerVar('bomberman_announce_bomber_console'))
      if int_option:
         es.cexec(int_userid, 'echo ' + remove_tags(func_lang_text('bomber', {}, str_player_lang)))
         if int_option in (2, 3):
            for int_loop_userid in es.getUseridList():
               if int_loop_userid <> int_userid and (int_option == 3 or int_team == es.getplayerteam(int_loop_userid)):
                  es.cexec(int_loop_userid, 'echo ' + remove_tags(func_lang_text('bomber all', dict_tokens, playerlib.getPlayer(int_loop_userid).get('lang'))))

      int_option_hurt = int(es.ServerVar('bomberman_bomber_trigger_hurt'))
      int_option_fire = int(es.ServerVar('bomberman_bomber_trigger_fire'))
      if int_option_hurt and int_option_fire:
         str_instruction_name = 'bomber fire hurt'
      elif int_option_hurt:
         str_instruction_name = 'bomber hurt'
      elif int_option_fire:
         str_instruction_name = 'bomber fire'
      else:
         str_instruction_name = ''
      if str_instruction_name:
         if int(es.ServerVar('bomberman_announce_bomber_trigger_chat')):
            es.tell(int_userid, '#multi', func_lang_text(str_instruction_name, '', str_player_lang))
         if int(es.ServerVar('bomberman_announce_bomber_trigger_console')):
            es.cexec(int_userid, 'echo ' + remove_tags(func_lang_text(str_instruction_name, '', str_player_lang)))


def finalize_bomber(int_userid):
   """
   Colors the player the bomber color
   Emits the bomber sound from the player
   """
   global str_last_sound
   global dict_players

   dict_current_player = dict_players[int_userid]
   dict_current_player['is_bomber'] = 1
   float_speed = float(es.ServerVar('bomberman_bomber_speed'))
   if float_speed <> 100:
      es.setplayerprop(int_userid, 'CBasePlayer.localdata.m_flLaggedMovementValue', float_speed / 100)
   dict_current_player['speed'] = float_speed
   int_color = int(es.ServerVar('bomberman_bomber_color'))
   int_alpha = int(es.ServerVar('bomberman_bomber_transparency'))
   if int_color or int_alpha <> 255:
      es.server.cmd('es_xfire %s !self addoutput \"rendermode 1\"' % int_userid)
      if int_color:
         es.server.cmd('es_xfire %s !self color \"%s\"' % (int_userid, dict_bomber_colors[int_color]['string']))
      if int_alpha <> 255:
         es.server.cmd('es_xfire %s !self alpha %s' % (int_userid, int_alpha))
      color_weapons(int_userid, int_color, int_alpha, 0)
   dict_current_player['color'] = int_color
   dict_current_player['alpha'] = int_alpha

   float_sound = float(es.ServerVar('bomberman_bomber_sound'))
   if float_sound:
      list_possible_sounds = list(tuple_sounds)
      if str_last_sound in list_possible_sounds:
         list_possible_sounds.remove(str_last_sound)
      es.emitsound('player', int_userid, random.choice(list_possible_sounds), '1.0', (100 - float_sound) / 100)


def explode_bomber(int_userid):
   """
   Explodes the player
   Ensures the player is dead
   Adds the point lost from the suicide to the player's score
   """
   for int_index in es.createentitylist('env_explosion'):
      es.server.cmd('es_xremove %s' % int_index)
   es.server.cmd('es_xgive %s env_explosion' % int_userid)
   es.server.cmd('es_xfire %s env_explosion addoutput \"iradiusoverride %s\"' % (int_userid, es.ServerVar('bomberman_explosion_radius')))
   es.server.cmd('es_xfire %s env_explosion addoutput \"imagnitude %s\"' % (int_userid, es.ServerVar('bomberman_explosion_magnitude')))
   es.setindexprop(es.ServerVar('eventscripts_lastgive'), 'CBaseEntity.m_hOwnerEntity', es.getplayerhandle(int_userid))
   es.server.cmd('es_xfire %s env_explosion explode' % int_userid)
   es.server.cmd('es_xfire %s env_explosion kill' % int_userid)
   float_sound = float(es.ServerVar('bomberman_explosion_sound'))
   if float_sound:
      es.emitsound('player', int_userid, 'weapons/hegrenade/explode%s.wav' % random.randint(3, 5), '1.0', (100 - float_sound) / 100)

   if not es.getplayerprop(int_userid, 'CBasePlayer.pl.deadflag'):
      es.sexec(int_userid, 'kill')

   dict_game_scores = es.createentitylist('game_score')
   int_game_score_index = es.getentityindex('game_score')
   if len(dict_game_scores) > 1:
      for int_index in dict_game_scores:
         if int_index <> int_game_score_index:
            es.server.cmd('es_xremove %s' % int_index)
   elif int_game_score_index == -1:
      es.server.cmd('es_xgive %s game_score' % int_userid)
   es.server.cmd('es_xfire %s game_score addoutput \"spawnflags 1\"' % int_userid)
   es.server.cmd('es_xfire %s game_score addoutput \"points 1\"' % int_userid)
   es.server.cmd('es_xfire %s game_score applyscore' % int_userid)


def remove_bomber(int_userid):
   """Removes the bomber status from the player"""
   global dict_players

   dict_current_player = dict_players[int_userid]
   dict_current_player['is_bomber'] = 0
   if dict_current_player['speed'] <> 100:
      es.setplayerprop(int_userid, 'CBasePlayer.localdata.m_flLaggedMovementValue', 1.0)
      dict_current_player['speed'] = 100
   int_color = dict_current_player['color']
   int_alpha = dict_current_player['alpha']
   if int_color or int_alpha <> 255:
      if int_color:
         es.server.cmd('es_xfire %s !self color \"255 255 255\"' % int_userid)
         dict_current_player['color'] = 0
      if int_alpha <> 255:
         es.server.cmd('es_xfire %s !self alpha 255' % int_userid)
         dict_current_player['alpha'] = 255
      color_weapons(int_userid, 0, 255, 1)
   dict_current_player['alpha']['weapons'][:] = []


def color_weapons(int_userid, int_color, int_alpha, int_allow_zero):
   """
   Sets the player's previous weapons to normal color and alpha
   Sets the player's current weapons to the player's color and alpha
   """
   global dict_players

   if int_color or int_alpha <> 255 or int_allow_zero:
      dict_current_player = dict_players[int_userid]
      int_player_handle = es.getplayerhandle(int_userid)
      list_to_remove = []
      dict_entity_list = es.createentitylist()
      float_entity_color = dict_bomber_colors[int_color]['int'] + (int_alpha * 16777216)

      for int_weapon_index in dict_current_player['weapons']:
         if not dict_entity_list.has_key(int_weapon_index):
            list_to_remove.append(int_weapon_index)
         else:
            if int_player_handle <> es.getindexprop(int_weapon_index, 'CBaseEntity.m_hOwnerEntity'):
               list_to_remove.append(int_weapon_index)
               es.setindexprop(int_weapon_index, 'CBaseEntity.m_nRenderMode', es.getindexprop(int_weapon_index, 'CBaseEntity.m_nRenderMode') | 1)
               es.setindexprop(int_weapon_index, 'CBaseEntity.m_nRenderFX', es.getindexprop(int_weapon_index, 'CBaseEntity.m_nRenderFX') | 256)
               es.setindexprop(int_weapon_index, 'CBaseEntity.m_clrRender', 4294967295)
      for int_weapon_index in list_to_remove:
         dict_current_player['weapons'].remove(int_weapon_index)

      for int_weapon_index in dict_entity_list:
         if dict_entity_list[int_weapon_index]['classname'][0:7] == 'weapon_' and int_player_handle == es.getindexprop(int_weapon_index, 'CBaseEntity.m_hOwnerEntity') and int_weapon_index not in dict_current_player['weapons']:
            dict_current_player['weapons'].append(int_weapon_index)
            es.setindexprop(int_weapon_index, 'CBaseEntity.m_nRenderMode', es.getindexprop(int_weapon_index, 'CBaseEntity.m_nRenderMode') | 1)
            es.setindexprop(int_weapon_index, 'CBaseEntity.m_nRenderFX', es.getindexprop(int_weapon_index, 'CBaseEntity.m_nRenderFX') | 256)
            es.setindexprop(int_weapon_index, 'CBaseEntity.m_clrRender', float_entity_color)


def remove_tags(str_text):
   """Removes #lightgreen, #green and #default tags from the supplied string"""
   return str_text.replace('#lightgreen', '').replace('#green', '').replace('#default', '')


def missing_lang(str_heading, dict_tokens, str_language):
   """Displays an error message because the languages .ini could not be found"""
   return 'BomberMan: bomberman_languages.ini not found'